package tcc.haruki.elgamal;

import java.math.BigInteger;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;

import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;

import tcc.haruki.domain.XTRElem;
import tcc.haruki.domain.xtr.XTRKey;
import tcc.haruki.domain.xtr.XTRParameters;
import tcc.haruki.elgamal.util.CriptografiaSimetricaImpl;
import tcc.haruki.elgamal.util.ICriptografiaSimetrica;
import tcc.haruki.exceptions.XTRInvalidKeyException;
import tcc.haruki.exceptions.XTRInvalidParametersException;
import tcc.haruki.exceptions.XTRNullParametersException;
import tcc.haruki.services.XTROperationService;
import tcc.haruki.services.impl.XTRDefaultOperationService;

public class ElGamal {
	private XTRParameters parameters;
	
	private XTRKey key;
	
	private XTROperationService xtrService;
	
	private ICriptografiaSimetrica protocoloAuxiliar;
	
	private int size;
	
	public void init() throws XTRNullParametersException, XTRInvalidParametersException, XTRInvalidKeyException {
		key = XTRKey.generateXTRKey(parameters);
		xtrService = new XTRDefaultOperationService(parameters);
		size = (this.key.getPublicKey().getP().bitLength() - 1)/8;
		protocoloAuxiliar = new CriptografiaSimetricaImpl();
	}
	
	public byte[] encrypt(byte[] msg, BigInteger y) {
		XTRElem newKey = xtrService.exponenciacaoSimples(key.getPublicKey().getH().get(0), y);
		
		byte[] chaveSimetrica = newKey.getByteRepresentation();
		protocoloAuxiliar.setChaveSimetrica(chaveSimetrica);
		
		try {
			return protocoloAuxiliar.encrypt(msg);
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public byte[] decrypt(byte[] cri, XTRElem el) {
		XTRElem newKey = xtrService.exponenciacaoSimples(el, key.getPrivateKey().getK());
		
		byte[] chaveSimetrica = newKey.getByteRepresentation();
		protocoloAuxiliar.setChaveSimetrica(chaveSimetrica);
		
		try {
			return protocoloAuxiliar.decrypt(cri);
		} catch (InvalidKeyException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalBlockSizeException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (BadPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchPaddingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	
	public void alo() {
		BigInteger y = new BigInteger("32423");
		
		byte[] msg = "alo xd".getBytes();
		
		XTRElem el = xtrService.exponenciacaoSimples(key.getPrivateKey().getTrg(), y);

		byte[] en = encrypt(msg, y);
		byte[] dec = decrypt(en, el);
		
		System.out.println(new BigInteger(msg));
		System.out.println(new BigInteger(en));
		System.out.println(new BigInteger(dec));
	}
	
	public static void main(String[] args) throws XTRNullParametersException, XTRInvalidParametersException, XTRInvalidKeyException {
		ElGamal elgamal = new ElGamal();
		elgamal.parameters = XTRParameters.generateParameters();
		elgamal.init();
		elgamal.alo();
	}
}